﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class HellShotgunBehaviour : CanShoot
{
    AudioSource audio;

    Vector2 barrelPos = new Vector2(0.5f, 0f);
    Vector2 barrelDir = Vector2.right;

    bool isLoaded = true;
    bool Reloading = false;

    DistanceJoint2D newJoint;

    public GameObject hook;

    float t = 0;
    float chargeHeat = 0f;

    bool held;

    BallisticsEmitter Ballistics;
    Transform muzzleTransform;
    ParticleSystem flash;
    AudioClip[] ShotSounds;
    GameObject BaseGun;
    FirearmBehaviour firearm;
    PhysicalProperties prop;

    public void Start()
    {

        Cartridge HellShotgun = ModAPI.FindCartridge("50 AE");
        HellShotgun.name = "Hell Shotgun";
        HellShotgun.Damage *= 1.25f;
        HellShotgun.StartSpeed *= 1.5f;
        HellShotgun.PenetrationRandomAngleMultiplier = 0f;
        HellShotgun.Recoil *= 1.25f;
        HellShotgun.ImpactForce *= 2f;

        BaseGun = ModAPI.FindSpawnable("Shotgun").Prefab;
        Ballistics = new BallisticsEmitter(this, HellShotgun);
        Ballistics.LayersToHit = BaseGun.GetComponent<FirearmBehaviour>().LayersToHit;
        muzzleTransform = Instantiate(Resources.Load<GameObject>("Prefabs/MuzzleSmoke"), transform).transform;
        flash = muzzleTransform.GetComponentInChildren<ParticleSystem>();
        audio = gameObject.AddComponent<AudioSource>();
        audio.volume = 0.75f;
        audio.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
        audio.outputAudioMixerGroup = Global.main.SoundEffects;

        firearm = BaseGun.GetComponent<FirearmBehaviour>();
        ShotSounds = firearm.ShotSounds;

        hook = transform.GetChild(0).gameObject;

        prop = hook.GetComponent<PhysicalBehaviour>().Properties;
        prop.SharpAxes = new SharpAxis[] { new SharpAxis(Vector2.right, -0.2f, 0.23f, false, true) };
    }

    public void Use()
    {
        t = 0f;
        held = true;
    }

    public void UseContinuous()
    {
        t += Time.deltaTime * 0.5f;
        chargeHeat = 1f;
    }

    void Update()
    {
        chargeHeat -= 0.5f;

        if (chargeHeat < 0f && held)
        {
            if (t <= 0.1f)
            {
                Shoot();
            } else if (!isLoaded && !Reloading)
            {
                StartCoroutine(Reload());
            }

            chargeHeat = 0f;
            t = 0f;
            held = false;
        }
        else
        {
            if(t >= 0.05f && !Reloading && isLoaded)
            {
                ShootHook();
            }
        }

    }

    public override void Shoot()
    {
        if (UserPreferenceManager.Current.TracerBullets)
        {
            Ballistics.MaxRange = Mathf.Clamp(firearm.Cartridge.StartSpeed / 4f, 4f, 8f);
            Ballistics.DoTraversal = true;
        }
        else
        {
            Ballistics.MaxRange = 1000f;
            Ballistics.DoTraversal = false;
        }

        for (int i = 0; i < firearm.BulletsPerShot * 1.5f; i++)
            Ballistics.Emit(BarrelPosition, new Vector2(BarrelDirection.x, BarrelDirection.y + Random.Range(-0.25f, 0.25f)));

        ExplosionCreator.CreatePulseExplosion(BarrelPosition, Ballistics.Cartridge.Recoil / 25f, Ballistics.Cartridge.Recoil / 3f, false);
        CameraShakeBehaviour.main.Shake(Ballistics.Cartridge.ImpactForce * firearm.BulletsPerShot, BarrelPosition);
        muzzleTransform.localScale = Vector3.one * Mathf.Clamp(Mathf.Pow(Ballistics.Cartridge.Damage * firearm.BulletsPerShot, 0.6f) / 4f, 0.25f, 2.2f);
        muzzleTransform.localScale = new Vector3(this.transform.lossyScale.x * muzzleTransform.localScale.x, muzzleTransform.localScale.y, muzzleTransform.localScale.z);
        muzzleTransform.position = BarrelPosition;
        muzzleTransform.right = BarrelDirection;
        audio.PlayOneShot(ShotSounds[Random.Range(0, ShotSounds.Length-1)]);
        flash.Play();
    }

    void ShootHook()
    {
        if (isLoaded)
        {
            audio.PlayOneShot(ModResources.harpoonLaunchSFX);
            Destroy(hook.GetComponent<FixedJoint2D>());
            hook.GetComponent<Rigidbody2D>().AddForce((hook.transform.right * transform.localScale.x) * 10f, ForceMode2D.Impulse);

            newJoint = hook.AddComponent<DistanceJoint2D>();
            newJoint.anchor = new Vector2(-0.235f, -0.015f);
            newJoint.connectedAnchor = new Vector2(0f, -0.015f);
            newJoint.connectedBody = gameObject.GetComponent<Rigidbody2D>();
            newJoint.maxDistanceOnly = true;
            newJoint.distance = 10f;

            JointCreated(newJoint);

            isLoaded = false;
        }
    }

    IEnumerator Reload()
    {
        Reloading = true;
        prop.SharpAxes = new SharpAxis[] { new SharpAxis(Vector2.right, -0.1f, 0.23f, false, false) };
        while (Vector2.Distance(gameObject.transform.position, hook.transform.position) > 0.3f)
        {
            newJoint.distance -= 0.1f;
            yield return new WaitForSeconds(0.005f);
        }
        newJoint.breakForce = 0f;
        hook.transform.localRotation = Quaternion.Euler(0f, 0f, 0f);
        hook.transform.localPosition = new Vector3(0.235f, -0.015f);
        FixedJoint2D joint = hook.AddComponent<FixedJoint2D>();
        joint.connectedBody = GetComponent<Rigidbody2D>();
        Reloading = false;
        isLoaded = true;
        audio.PlayOneShot(ModResources.harpoonReloadSFX);
        prop.SharpAxes = new SharpAxis[] { new SharpAxis(Vector2.right, -0.1f, 0.23f, false, true) };
    }

    void JointCreated(DistanceJoint2D joint)
    {
        EnergyWireBehaviour energyWireBehaviour = joint.gameObject.AddComponent<EnergyWireBehaviour>();
        energyWireBehaviour.WireColor = Color.black;
        energyWireBehaviour.WireMaterial = Resources.Load<Material>("Materials/Wire");
        energyWireBehaviour.WireWidth = 0.025f;
        energyWireBehaviour.typedJoint = joint;
    }

    public override Vector2 BarrelPosition { get { return transform.TransformPoint(barrelPos); } }
    public Vector2 BarrelDirection { get { return transform.TransformDirection(barrelDir) * transform.localScale.x; } }
}
